@@ -1,9 +1,8 @@
 #!/bin/sh -f
 # ssh
-cd $COSMOSTOP/UserHook/DisPara/FleshBasic/
-#  to synclonize with other submission
-echo "ok" > ./fleshbasic.__
+cd $COSMOSTOP/UserHook/DisParaRescue/FleshBasic/
 source ./setupenv.sh $0
+source ../Smash/setupenv.sh $0
 #    __ will be replaced by cpu# by execflesh_all or _one
 NUMB=__
 export NUMB
