@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/sh -f
 if [  $# != 1 ];  then
     echo 'Usage ../execflesh.sh [sge|ssh]'
     echo 'This must be invoked from FleshHist'
@@ -19,12 +19,6 @@ else
     exit
 fi
 
-
-echo  "Assume the file specified by SkeletonFile at Skeleton making time is"
-echo  "$sparam"
-echo  "Enter y if it is so."
-read yesno
-test "x$yesno" != "xy" && exit
 ##   next argument is for avoiding QA in the script
 source ../Smash/setupenv.sh $0
 source ./setupenv.sh  $0
