@@ -13,6 +13,9 @@ c
 c
       real*8 po, f, ecm, cosa, lam
       integer  i, charge, subcode
+      logical  try
+
+      try = .true.
 c
 c           make 3 ptcls; neu_e, neu_mu, e
       subcode = -pj.charge
@@ -29,47 +32,53 @@ c                neue trino opposit.
       else
          po=polari
       endif
+      do while ( try )
 c           sample energy in f=2e*/mmu: e* is at muon rest sytem
 c           of neue
-      call csampNeueEMu(f)
-      ecm=f*pj.mass/2
+         call csampNeueEMu(f)
+         ecm=f*pj.mass/2
 c              sample decay angle of neue at muon rest system
 c            (1+Pcos)dcos
-      call ksampLin(po, 1.d0, -1.d0, 1.d0, cosa)
+         call ksampLin(po, 1.d0, -1.d0, 1.d0, cosa)
 c              set random momentum about azimuth (px,py,pz)
-      call cpCos2pxyz(cosa, ecm, a(1).fm)
-      a(1).fm.p(4) = ecm
+         call cpCos2pxyz(cosa, ecm, a(1).fm)
+         a(1).fm.p(4) = ecm
 c                 since inclusive, no conservation tried
 c          neumu; sample energy in f=2e*/mmu: e* is at muon rest sytem
-      call csNeumuEMu(f)
-      ecm=f*pj.mass/2
+         call csNeumuEMu(f)
+         ecm=f*pj.mass/2
 c              sample decay angle of neumu at muon rest system
 c             (1+lambda P cos) dcos with
 c             lambda = (1-2f)/(3-2f)
-      lam=(1.-2*f)/(3.-2*f)
-      call ksampLin(lam*po, 1.d0, -1.d0, 1.d0, cosa)
+         lam=(1.-2*f)/(3.-2*f)
+         call ksampLin(lam*po, 1.d0, -1.d0, 1.d0, cosa)
 c             set px,py,pz
-      call cpCos2pxyz(cosa, ecm, a(2).fm)
-      a(2).fm.p(4) = ecm
+         call cpCos2pxyz(cosa, ecm, a(2).fm)
+         a(2).fm.p(4) = ecm
 
 c               electron ; paralell to neu_mu
 c         x^2(3-2x + (1-2x)Pcos) dxdcos
 c            first integrate by cos and get energy distribution
 c         x^2(3-2x) dx: this is the same as  neu_e energy (x=2E/Mmu)
-      call csNeumuEMu(f)
+         call csNeumuEMu(f)
 c         angluar distribution is
 c        ( 1 + lam* cos ) dcos type
 c
-      lam = (1.-2*f)/(3.-2*f) *po
-      call ksampLin(lam, 1.d0, -1.d0, 1.d0, cosa)
+         lam = (1.-2*f)/(3.-2*f) *po
+         call ksampLin(lam, 1.d0, -1.d0, 1.d0, cosa)
 c             set px,py,pz
-      call cpCos2pxyz(cosa, ecm, a(3).fm)
-      a(3).fm.p(4) = ecm
+         call cpCos2pxyz(cosa, ecm, a(3).fm)
+         a(3).fm.p(4) = ecm
 
 c
-      np = 3
+         np = 3
 c               boost to lab.
-      do i = 1, np
-         call cibstPol(i, pj, a(i), a(i) )
+         do i = 1, np
+            call cibstPol(i, pj, a(i), a(i) )
+         enddo
+c         since approximation here is to assume Me=0
+c         we get Ee < Me sometimes (normally 5/10^5 or less )
+c         so we reject such an event
+         try =  a(3).fm.p(4) .le. a(3).mass
       enddo
       end
