@@ -34,11 +34,11 @@ c    &&&&&&&&&&&&&&
 c
 
       integer irsave(2)
-c
+      logical dodpm
       real*8    ELAB
       integer   IDP, NTMASS, NTCHAR, NPMASS, NPCHAR
       common /dpmjetcom/ ELAB, IDP, NTMASS, NTCHAR, NPMASS,NPCHAR
-     *  ,irsave
+     *  ,irsave, dodpm
 c    &&&&&&&&&&&
 c
       record /ptcl/ pj  ! input. projectile particle
@@ -58,7 +58,7 @@ c/////////
       integer  NEVENT, ICASCA
       COMMON /DTEVNO/ NEVENT,ICASCA
 
-      logical dodpm, first
+      logical first
       save first
       data first /.true./
 c////////////
@@ -163,7 +163,10 @@ c                  if(loopc .gt. 100) then    ! probably useless.
      *                    ' loopc=',loopc
                      call cpdpmjetinp
                      call cerrorMsg(
-     *              'dpmjet3 too many loop in cdpmjet',0)     
+     *              'dpmjet3 too many loop in cdpmjet',1)     
+c                      no ptcl produced
+                     call cdpmNoPtcl
+                     icon = 0
                   endif
                endif
             enddo
@@ -210,13 +213,16 @@ c            print current dpmjet input
       character*100 msg
 c    &&&&&&&&&&&&&&
       integer irsave(2)
-
+      logical dodpm
       real*8    ELAB
       integer   IDP, NTMASS, NTCHAR, NPMASS, NPCHAR
       common /dpmjetcom/ ELAB, IDP, NTMASS, NTCHAR, NPMASS,NPCHAR
-     *   ,irsave
+     *   ,irsave, dodpm
 c    &&&&&&&&&&&
-c
+      character*16 model
+      call cqActiveMdl(model)
+      write(msg, *) ' dodpm=',dodpm, ' activemodel=', model
+      call cerrorMsg(msg, 1)
       write(msg, *) ' dpmjet input: IDP=', IDP, ' Elab=',ELAB
       call cerrorMsg(msg, 1)
       write(msg, *) ' target: A=', NTMASS, ' Z=',NTCHAR,
@@ -323,6 +329,20 @@ c             residual nucleus; neglect
 c     -------------- rotate
       call crot3mom(pj, a, ntp)
       end
+      subroutine cdpmNoPtcl
+c         clear no. of produced partilce
+      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
+*
+*
+      PARAMETER (NMXHKK=90000)
+
+      COMMON /DTEVT1/ NHKK,NEVHKK,ISTHKK(NMXHKK),IDHKK(NMXHKK),
+     &                JMOHKK(2,NMXHKK),JDAHKK(2,NMXHKK),
+     &                PHKK(5,NMXHKK),VHKK(4,NMXHKK),WHKK(4,NMXHKK)
+
+      NHKK= 0
+      end
+
       SUBROUTINE DT_USRHIS(MODE)
       implicit none
       integer MODE
