@@ -6,5 +6,7 @@ c       fortran 90 version  1D histogram
          real, allocatable ::  dnw(:)
          real, allocatable ::  dndx(:)
          real, allocatable ::  mean(:)
+         real, allocatable ::  xforI(:)
+         real, allocatable ::  integ(:)
       end type
 
