@@ -4,12 +4,14 @@ c            (Not work under Absoft fortran 90)
 c      Usage:  kwhisti:   insatnciate one histogram
 c              kwhistc:   clear histogram area
 c              kwhist:    take histogram
+c              kwhists0:  specify integral to be from small or large
 c              kwhists:   compute statistical result.
 c                        This can be used more than once 
 c                        with a differennt normalization factor
 c                        for the same histogram.   
 c                 Therefore, in bin2ascii you may use inorm=-1 to 
 c                    keep the previous value.
+
 c              kwhistp:   print statistical result or binary write histogram result
 c                         by calling kwhistpr or kwhistw
 c              kwhistw:  write histogram with binary format
@@ -31,7 +33,8 @@ c
       subroutine kwhisti(h, ixmin, ibinORxmax, inbin, itklg )
       implicit none
 c         instanciate
-      integer inbin  ! input. request inbin histogram area
+      integer inbin  ! input. request inbin histogram area ( actual bin number
+                     ! will be <= inbin-1. ).
       real ixmin     ! input. xmin. not in log even if log10(variable) is taken
                      !         see itklg 
       real ibinORxmax  ! input. bin or ixmax. depends on itklg.
@@ -85,6 +88,8 @@ c
       real*8 isumw
       real dx, normf
       real xmin, xmax
+      logical oldfmt/.false./, old
+      integer iofl, iufl
       integer bfnow  !  binary  write file no.
       integer bfnor  !  binary  read file no.
       integer icon   !  0; binary read was successful
@@ -118,25 +123,33 @@ c
 c   -------------------------
       integer itempx, itempv, nstr
       character*96  dirstr
+      real*8 integral, xl
       integer klena
-      save normf
+      integer from, fromwhich/0/   ! spcify integration direction. 
+                   !  0--> from small.  !=0 --> from  large.
+      save normf, fromwhich, oldfmt
 
       if( h%c%init .eq. 'initend') then
          write(0, *) '1D hist already instanciated'
          write(0, *) ' title=',h%c%title
-         write(0, *) ' categorye=',h%c%categ
+         write(0, *) ' category=',h%c%categ
          write(0, *) ' id=',h%c%id
          stop 9999
       else
          h%c%init = 'initend'
       endif
 
-      h%x%nhist = inbin
+      h%x%nhist = inbin 
       allocate( h%xw(inbin) )
       allocate( h%dnw(inbin) )
       allocate( h%mean(inbin) )
       allocate( h%dndx(inbin) )
+      allocate( h%xforI(inbin) )
+      allocate( h%integ(inbin) )
+
+
       h%x%tklg  =( itklg - (itklg/2)*2  ) .ne. 0
+c     h%x%tklg  = bit(0, itklg)   ! etc may be used; logical bit needed
       h%x%cent  =( (itklg/2)*2 - (itklg/4)*4 ) /2    ! integer
       h%x%ufl  = ( (itklg/4)*4 - (itklg/8)*8 ) .ne. 0
       h%x%ofl  = ( (itklg/8)*8 - (itklg/16)*16 ) .ne. 0
@@ -150,9 +163,9 @@ c   -------------------------
             stop 99999
          else
             if( h%x%cent .eq. 1 ) then
-               ndiv= inbin - 1
+               ndiv= inbin - 2
             else
-               ndiv = inbin
+               ndiv = inbin-1
             endif
             if(h%x%tklg) then
                h%x%bin = log10(ibinORxmax/ixmin)/ndiv
@@ -193,11 +206,16 @@ c
 c    ************************
       entry kwhistc(h)
 c    ************************
+
       do i = 1, h%x%nhist
          h%xw(i) = 0.
          h%dnw(i) = 0.
       enddo
       return
+
+      entry kwhistfmt(old)
+      oldfmt=old
+      return
 c    *************************
       entry kwhist( h, x, w )
 c    *************************
@@ -213,18 +231,21 @@ c         neglect this data
 
          if(i .le. 0 .and. h%x%ufl ) then
             i = 1
-         elseif(i .gt. h%x%nhist .and. h%x%ofl ) then
-            i = h%x%nhist
+         elseif(i .ge. h%x%nhist .and. h%x%ofl ) then
+            i = h%x%nhist-1
          endif
-                                                        
-         if(i .ge. 1 .and.  i  .le. h%x%nhist )  then
+         if(i .ge. 1 .and.  i  .lt. h%x%nhist )  then
             h%xw(i) = h%xw(i)  +  x*w
             h%dnw(i) = h%dnw(i) + w
+            h%mean(i) = h%mean(i) + 1.0
          endif
 
       endif
       return
-
+c     ******************
+      entry kwhists0(from)
+      fromwhich = from
+      return
 c     ***********************
       entry kwhists( h, inorm )
 c     ************* take statistics
@@ -237,18 +258,17 @@ c         if inorm = -1.0, use alredy fixed one.
       do while( h%x%imin .lt. h%x%nhist .and.  h%dnw(h%x%imin) .eq. 0.) 
          h%x%imin = h%x%imin + 1
       enddo
-
-      h%x%imax = h%x%nhist
-      do while (h%x%nhist .gt. 1 .and.  h%dnw(h%x%imax) .eq.  0.)  
+      h%x%imax = h%x%nhist-1
+      do while (h%x%imax .gt. 1 .and.  h%dnw(h%x%imax) .eq.  0.)  
          h%x%imax = h%x%imax -1
       enddo
 
 
-
-      h%x%sumw = 0
+      isumw = 0.
       do i = h%x%imin, h%x%imax
-         h%x%sumw = h%x%sumw +  h%dnw(i)
+         isumw = isumw +  h%dnw(i)
       enddo
+      h%x%sumw =  isumw
 
       if(h%c%norm .eq. 0. .and.  h%x%sumw  .gt. 0.) then
         normf = h%x%sumw 
@@ -258,32 +278,68 @@ c         if inorm = -1.0, use alredy fixed one.
          normf = h%c%norm
       endif
 
-c        bin center value
+c        bin center value and left boundary
       if( h%x%tklg ) then
          xx =10**(h%x%xm + h%x%bin/2.) * h%x%inc**(h%x%imin-1)
+         xl =10**h%x%xm * h%x%inc**(h%x%imin-1)
       else
          xx = h%x%xm +   h%x%bin/2 + h%x%inc*(h%x%imin-1)
+         xl = h%x%xm + h%x%inc*(h%x%imin-1)
       endif
 
+         
+      if(fromwhich .eq. 0 ) then
+         integral = 0.
+      else
+         integral = isumw
+      endif
+      
+
       dx = h%x%bin      
       do i = h%x%imin, h%x%imax
          if( h%x%tklg ) then
             dx  = 10.0**(h%x%xm + i * h%x%bin) -
      *            10.0**(h%x%xm + (i-1)*h%x%bin)
          endif
-         if(h%dnw(i) .eq. 0) then
-            h%mean(i) = xx
-         else
-            h%mean(i) = h%xw(i)/h%dnw(i)
-         endif
-
+c         if( inorm .eq. -1.0) then
+c            data is probbly read from file so mean exists
+ccc            if( h%dnw(i) .eq. 0) then
+ccc               h%xw(i) = xx
+ccc            else
+ccc               h%xw(i) = h%mean(i)* h%dnw(i)
+ccc            endif
+ccc         else
+cc            if(h%dnw(i) .eq. 0) then
+cc               h%mean(i) = xx
+cc            else
+cc               h%mean(i) = h%xw(i)/h%dnw(i)
+cc            endif
+cc         endif
+c
+         h%xforI(i) = xl
+         h%integ(i) = integral/normf
          h%dndx(i) = h%dnw(i)/dx/normf
          if( h%x%tklg ) then
             xx = xx * h%x%inc
+            xl = xl * h%x%inc
          else
             xx = xx + h%x%inc
+            xl = xl + h%x%inc
+         endif
+         if(fromwhich .eq. 0) then
+            integral = integral + h%dnw(i)
+         else
+            integral = integral - h%dnw(i)
          endif
       enddo
+c      if(.not. oldfmt) then
+         h%xforI(h%x%imax+1) =xl
+         h%integ(h%x%imax+1) =integral/normf
+c      endif
+      h%dndx(h%x%imax+1) =0.
+      h%dnw(h%x%imax+1) =0.
+cc      h%mean(h%x%imax+1) =xx
+      h%mean(h%x%imax+1) =0.
       return
 c     ********************
       entry kwhistev(h, evno)
@@ -324,16 +380,42 @@ c
 c     *********************
       entry kwhistpr( h, fno )
 c     ****************print  hist
-      isumw = h%x%sumw
 
+c       next lines for getting sumw is needed
+c       sumv read from ascii file has some error
+c       and results in 10^-5 diff. eventually
+c
+      isumw = 0.
+      do i = h%x%imin, h%x%imax
+         isumw =  isumw + h%dnw(i)
+      enddo
+      h%x%sumw =  isumw 
+    
+c        next must be given ; normf is undef. when
+c        ascii file is read.
+      if(h%c%norm .eq. 0. .and.  h%x%sumw  .gt. 0.) then
+         normf = isumw
+      elseif(h%c%norm .le. 0. ) then
+         normf = 1.0
+      else
+         normf = h%c%norm
+      endif
+c      ......................
       if( h%x%tklg ) then
          xx = 10.0**(h%x%xm + h%x%bin/2.0) * h%x%inc**(h%x%imin-1)
       else
          xx = h%x%xm + h%x%bin/2. + h%x%inc*(h%x%imin-1)
       endif
 c        header
+      iufl = 0
+      if(h%x%ufl) iufl = 1
+      iofl = 0
+      if(h%x%ofl) iofl = 1
+ 
       if(fno .lt. 0) then
-         write(*, '(a, i3)') '#hist1 ', h%c%eventno
+         write(*, '(a, i5, i7, 3i3, g15.6)') 
+     *     '#hist1 ', h%c%eventno, h%x%nhist,
+     *     h%x%cent, iufl, iofl, h%x%bin 
          write(*, '(a,a)') '#t ', h%c%title(1:klena(h%c%title))
          write(*, '(a,a)') '#c ', h%c%categ(1:klena(h%c%categ))
 
@@ -347,9 +429,14 @@ c        header
          itempv = 0
          if( h%c%logv) itempv = 1
          write(*,'(a, 2i3)')'#l ',  itempx, itempv
-         write(*, '(a, 1p2E11.3)')'#n ', isumw, normf
+         write(*, '(a, 1pE15.8, g15.6)')'#n ', isumw, normf
+         write(*, '(a,2i6,2g15.6)')
+     *   '#o ', h%x%imin, h%x%imax,
+     *          h%x%xm, h%x%inc
       else
-         write(fno, '(a,i3)') '#hist1 ', h%c%eventno
+         write(fno, '(a, i5, i7, 3i3, g15.6)') 
+     *     '#hist1 ', h%c%eventno, h%x%nhist,
+     *     h%x%cent, iufl, iofl, h%x%bin 
          write(fno, '(a,a)') '#t ', h%c%title(1:klena(h%c%title))
          write(fno, '(a,a)') '#c ', h%c%categ(1:klena(h%c%categ))
 
@@ -363,7 +450,10 @@ c        header
          itempv = 0
          if( h%c%logv) itempv = 1
          write(fno,'(a,2i3)') '#l ',  itempx, itempv
-         write(fno, '(a, 1p2E11.3)') '#n ', isumw, normf
+         write(fno, '(a, 1pE15.8, g15.6)') '#n ', isumw, normf
+         write(fno, '(a,2i6,2g15.6)')
+     *   '#o ', h%x%imin, h%x%imax,
+     *          h%x%xm,  h%x%inc
       endif
 
 c          directory where data is saved
@@ -375,26 +465,36 @@ c          directory where data is saved
          write(fno,'(a, a)') '#d ', dirstr(1:nstr)
       endif
 
-      dx = h%x%bin      
+         
+
       do i = h%x%imin, h%x%imax
-         if( h%x%tklg ) then
-            dx  = 10.0**(h%x%xm + i * h%x%bin) -
-     *            10.0**(h%x%xm + (i-1)*h%x%bin)
-         endif
          if(fno .lt. 0) then
-            write(*, '(i5, 1p6E11.3)')   i,
-     *       xx,  h%dndx(i), h%dnw(i), isumw,  h%mean(i), dx
+            write(*, '(i5, 1p6E13.5)')   i,
+     *      xx,  h%dndx(i), h%dnw(i), h%mean(i),  h%xforI(i), h%integ(i)
          else
-            write(fno, '(i5, 1p6E11.3)')  i,
-     *       xx,  h%dndx(i), h%dnw(i),  isumw, h%mean(i), dx 
+            write(fno, '(i5, 1p6E13.5)')  i,
+     *      xx,  h%dndx(i), h%dnw(i), h%mean(i),
+     *      h%xforI(i), h%integ(i)
          endif            
-         isumw = isumw -  h%dnw(i)
+
          if( h%x%tklg ) then
             xx =  xx * h%x%inc 
          else
             xx = xx + h%x%inc
          endif
       enddo
+
+
+      if(fno .lt. 0) then
+         write(*, '(i5, 1p6E13.5)')    h%x%imax+1,
+     *     xx,  0.,  0., 0.,
+     *      h%xforI( h%x%imax+1), h%integ( h%x%imax+1)
+      else
+         write(fno, '(i5, 1p6E13.5)')  h%x%imax+1,
+     *        xx,  0.,  0., 0.,
+     *      h%xforI( h%x%imax+1), h%integ( h%x%imax+1)
+      endif            
+      
 c       trailer
       if(fno .lt. 0) then
          write(*,'(7i3)')  0,0,0,0,0,0, 0
@@ -409,7 +509,8 @@ c       binary write of h to bfnow
       write(bfnow) '#hist1'
       write(bfnow) h%x%nhist
       write(bfnow) h%x, h%c
-      write(bfnow) h%xw, h%dnw, h%mean, h%dndx
+      write(bfnow) h%xw, h%dnw, h%mean, h%dndx,
+     *             h%xforI, h%integ
       return
 c     *********************
       entry kwhistr(h, bfnor, icon)
@@ -417,12 +518,24 @@ c     ********************
 c        #hist1 must be read outside
 c
       read(bfnor, end =222)  nbin 
+
       allocate( h%xw(nbin) )
       allocate( h%dnw(nbin) )
       allocate( h%mean(nbin) )
       allocate( h%dndx(nbin) )
+c      if(.not. oldfmt) then
+         allocate( h%xforI(nbin) )
+         allocate( h%integ(nbin) )
+c      endif
+
       read(bfnor, end=222)  h%x, h%c
-      read(bfnor, end= 222)  h%xw, h%dnw, h%mean, h%dndx
+      if(oldfmt) then
+         read(bfnor, end= 222)  h%xw, h%dnw, h%mean, h%dndx
+      else
+        read(bfnor, end= 222)  h%xw, h%dnw, h%mean,
+     *   h%dndx, h%xforI, h%integ
+
+      endif
       icon = 0
       return
  222  continue
@@ -435,7 +548,13 @@ c     ***************
 c      deallocate histogram area
 c
       h%c%init = ' '
-      deallocate( h%xw, h%dnw,  h%mean,  h%dndx, stat=dealloc)
+c      if(.not. oldfmt) then
+         deallocate( h%xw, h%dnw,  h%mean,  h%dndx, 
+     *            h%xforI, h%integ, stat=dealloc)
+c      else
+c         deallocate( h%xw, h%dnw,  h%mean,  h%dndx, 
+c     *     stat=dealloc)
+c      endif
       if(dealloc .ne. 0) then
          write(0,*) ' dealloc failed =',dealloc
          stop 12345
@@ -460,6 +579,10 @@ c           not yet initialized.
          allocate( h%dnw(nbin) )
          allocate( h%mean(nbin) )
          allocate( h%dndx(nbin) )
+c         if(.not. oldfmt) then
+            allocate( h%xforI(nbin) )
+            allocate( h%integ(nbin) )
+c         endif
          h%c%init = 'initend'
       endif
       h%x = h1%x
@@ -506,5 +629,61 @@ c
       endif
       end
 
-      
+      integer function kwhistIxy(h, x, y, n) 
+      implicit none
+c     ******************
+      include 'Z90histc.h'
+      include 'Z90histo.h'
+      include "Z90hist1.h"
+      type(histogram1) h
+c     ==================
+      integer n  ! input size of x,y
+      real*8 x(n), y(n) ! output, (normalized) 
+                      ! integral value y at x (i=1, m)
+      integer m ! function value. number of data points in x,y
+                ! if number of x,y must be > n, m=-1 will be returned.
+                ! (error)
+      integer i
 
+      m = h%x%imax - h%x%imin +2
+      if(m .gt. n) then
+         m = -1
+      else
+         m = 0
+         do i = h%x%imin,  h%x%imax + 1
+            m = m + 1
+            x(m) = h%xforI(i)
+            y(m) = h%integ(i)
+         enddo
+      endif
+      kwhistIxy = m
+      end
+      integer function kwhistxy(h, x, y, n) 
+      implicit none
+c     ******************
+      include 'Z90histc.h'
+      include 'Z90histo.h'
+      include "Z90hist1.h"
+      type(histogram1) h
+c     ==================
+      integer n  ! input size of x,y
+      real*8 x(n), y(n) ! output, normalzied dn/dx value y at x (i=1, m)
+                      ! x is bin center.
+      integer m ! function value. number of data points in x,y
+                ! if number of x,y must be > n, m=-1 will be returned.
+                ! (error)
+      integer i
+
+      m = h%x%imax - h%x%imin + 1
+      if(m .gt. n) then
+         m = -1
+      else
+         m = 0
+         do i = h%x%imin,  h%x%imax
+            m = m + 1
+            x(m) = h%xw(i)
+            y(m) = h%dndx(i)
+         enddo
+      endif
+      kwhistxy = m
+      end
