@@ -59,10 +59,12 @@ c	(->	--------------------------------------------
                                !    It can by binary or ascii file depending on Trace value.
 	logical ThinSampling  !1  if F, thinsampling is not tried. if T, alla Hillas thinning. Don't use with
                               !    the  skeleton/flesh method 
-        real*8  EthinRatio(2) !2  if ThinsSamplig == T, thin sampling is performed if the energy of a particle is
+        real*8  EthinRatio(4) !2  if ThinsSamplig == T, thin sampling is performed if the energy of a particle is
                               !  between (EthinRatio(2)$\sim$EthinRatio(1))* PrimaryEnergy(/nucleon)
                               !   (=Ethin(2)$sim$Ethin(1)) ( EtinRatio(1)$>$ 0).
                               !  If EthinRatio(1) $<$ 0, Ethin will be |EthinRatio| (GeV). 
+                              !  (1),(2) is for e/g. (3),(4) is for mu/hadron.  if(3)(4) are not given, 
+                              !  (3)=(1)/10 and (4)=(2)/10 are used.
         logical TimeStructure !1  If T,  time information is computed
 	integer HowGeomag     !2  if 1, no magnetic field until first coll. \newline
                               !      2, mag.f always exists. If Reverse not=0, use this. \newline
