@@ -292,6 +292,15 @@ c           thin sampling
          else
             Ethin(1) = EthinRatio(1)*ergpn
          endif
+         if(EthinRatio(3) .lt. 0.) then
+            Ethin(3) = abs(EthinRatio(3))
+         elseif(EthinRatio(3) .eq. 0.) then
+            Ethin(3) = Ethin(1)/10.
+         else
+            Ethin(3) = EthinRatio(3)*ergpn
+         endif
+
+
 c         if(EthinRatio(2) .lt. 0.) then
 c            Ethin(2) = abs(EthinRatio(2))
 c         else
@@ -299,6 +308,11 @@ c            Ethin(2) = EthinRatio(2)*ergpn
 c         endif
 c             maximum weight above which no thinning
           Ethin(2) =  EthinRatio(2)
+          if( EthinRatio(4) .eq. 0.) then
+             Ethin(4) =  Ethin(2)/10.
+          else
+             Ethin(4) = EthinRatio(4)
+          endif
 c///////////////////
 c          write(0, *) ' ****** Ethin1,2=', Ethin
 c          write(0, *) ' ergpn=', ergpn, ' EthinRatio=',EthinRatio
@@ -307,6 +321,8 @@ c////////////////
       else
            Ethin(1) = 0.
            Ethin(2) = 0.
+           Ethin(3) = 0.
+           Ethin(4) = 0.
       endif
       end
       subroutine cexamParam
