@@ -127,12 +127,14 @@ c
                    ! in case of ThinSampling, this may be <= Nproduced. 
 
       record /track/ aTrack  
-      integer i, icon
-      
+      integer i
+      integer loc1
 
 
       aTrack = iTrack
       npush = 0
+      call cgetCurrentStackPos(loc1)  ! upto loc1 is  toalready filled 
+
       do i =  n, 1, -1          ! move leading ptcl first
 #ifdef SUBSTREC
          aTrack.p = pw(i)
@@ -146,36 +148,53 @@ c
 
 c               reset direction cos and related stuffs
          call cresetDirec(aTrack)
-         if(ThinSampling) then
-            call csetThinwgt(iTrack, aTrack,  icon)
-         else
-            icon = 0
-         endif
-         if(icon .eq.0) then
+
+
 #if LABELING == 1
 c
 c              whennever an interaction occur,  update labelcounter
 c             if info>0, clear the timer and infor counters.
 c
-            if(aTrack.info .gt. 0) then
+         if(aTrack.info .gt. 0) then
 ccc               aTrack.info = 0
 ccc               aTrack.t = 0.
-            endif
-            Labelcounter = Labelcounter + 1
-            aTrack.label = Labelcounter
+         endif
+         Labelcounter = Labelcounter + 1
+         aTrack.label = Labelcounter
 #elif LABELING == 2
 c            the above simple counter may be replaced by the
 c            next sophisticated one. 
 c            the same one is in the 1dim mode move routine below.
 c
-            call ctrickycount(iTrack, aTrack, pw, i)
+         call ctrickycount(iTrack, aTrack, pw, i)
 c
 #endif
-            npush = npush + 1
-            call cpush(aTrack)
-         endif
+         npush = npush + 1
+         call cpush(aTrack)
       enddo 
+      if(ThinSampling .and. npush .gt. 0 ) then
+         call cthinStack(loc1+1, npush, iTrack)
+      endif
       end
+      subroutine cthinStack(stackloc, n, iTrack)
+#include  "Zcode.h"
+#include  "Ztrackp.h"
+#include  "Ztrack.h"
+#include  "Ztrackv.h"
+#include  "Zstackv.h"
+c 
+      integer stackloc  ! first loc of stack where tracks of current int.
+      integer n
+      record /track/ iTrack  ! input. incident ptcl. of the interaction
+      integer nout
+
+      call cthinning(stack(stackloc), n, iTrack, nout)
+      call cresetStackPos(stackloc-1+nout)
+      end
+
+
+
+
 #if LABELING == 2
 c       ************************************
 c             This routine updates label counters.
@@ -264,14 +283,16 @@ c          1 dimensional case.
       integer  npush   !  output.  actuall number of ptcls put in stack.
       real*8 temp, p
 
-      integer i, icon
+      integer i
       
       record /track/ aTrack 
-
+      integer loc1
 
 
       aTrack = iTrack
 
+      call cgetCurrentStackPos(loc1)
+
       do i =  n, 1, -1          ! move leading ptcl last
 #ifdef SUBSTREC
          aTrack.p = pw(i)
@@ -300,34 +321,30 @@ c            call cresetMom(aTrack)  which is
             call cgetZenith(aTrack, aTrack.vec.coszenith)
 
 
-            if(ThinSampling) then
-               call csetThinwgt(iTrack, aTrack, icon)
-            else
-               icon = 0
-            endif
-            if(icon .eq. 0) then
 #if LABELING == 1
 c                  whennever secondary particles are generated,
 c               each of them get an updated label cocunter
 c               if the particle has crossed the highest level
 c               (info > 0),  timer and info counter is cleared
 c
-               if( aTrack.info .gt. 0) then
+            if( aTrack.info .gt. 0) then
 ccc                  aTrack.info = 0
 cc                  aTrack.t = 0.
-               endif
-               Labelcounter = Labelcounter + 1
-               aTrack.label = Labelcounter
+            endif
+            Labelcounter = Labelcounter + 1
+            aTrack.label = Labelcounter
 #elif LABELING == 2
 c                   this may be used if a tricky count is needed
 c               in stead of above  counting
-               call ctrickycount(iTrack, aTrack, pw, i)
+            call ctrickycount(iTrack, aTrack, pw, i)
 #endif
-               npush = npush + 1  
-               call cpush(aTrack)
-            endif
+            npush = npush + 1  
+            call cpush(aTrack)
          endif
       enddo
+      if(ThinSampling .and. npush .gt. 0 ) then
+         call cthinStack(loc1+1, npush, iTrack)
+      endif
       end
 
 c     ****************************************************************
@@ -366,130 +383,4 @@ cc     *    MovedTrack.p.code .le. khvymax ) then
 #endif
       endif
       end
-c     ***************************
-      subroutine csetThinwgt(iTrack, aTrack, icon)
-      implicit none
-#include  "Zcode.h"
-#include  "Ztrack.h"
-#include  "Ztrackp.h"
-#include  "Ztrackv.h"
-#include  "Zincidentv.h"
-#include  "Zelemagp.h"
-
-      record /track/ iTrack   ! input  parent particle
-      record /track/ aTrack  ! input   child particle
-c                            ! output.    wgt
-      integer icon           ! output.  0 if this is not tobe discarded
-                             !          1 if this is to be discarded
-      real*8 u, p 
-      real*8 iergpn, aergpn
-      logical dothin
-      real*8 dd, rhoE
-      real*8 cvh2den
-      real*8 Z1, E0
-      integer ii, jj  ! ii is for incident and jj is for child; for e/g 1,  for mu/had 3
-      data Z1/-1./, E0/-1./
-      save Z1, E0, rhoE, dd
-      
-      if(IncidentCopy.p.code .eq. kphoton  .and.
-     *    PhotoProd )  then
-c            photon  primary and muon is interested
-c            so thinsamling must be carfull
-c            we apply thinning only if current depth is
-c            > 120 g/cm2 from the first col. point.
-         if(Z1 .ne.  Zfirst.pos.depth  .or.
-     *      E0 .ne.  IncidentCopy.p.fm.p(4) ) then
-            Z1 = Zfirst.pos.depth 
-            E0 = IncidentCopy.p.fm.p(4) 
-            if( LpmEffect ) then
-               rhoE=cvh2den( Zfirst.pos.height )* 1.e-3 * 
-     *             IncidentCopy.p.fm.p(4) 
-               if(rhoE .lt. 1.e6)  then
-                  dd = 200.
-               else
-                  dd =min( 200.* sqrt(rhoE/1.e6), 1000.d0)
-               endif
-            else
-               dd = 200.
-            endif
-         endif
-
-         if(  MagBrem .eq. 2 .and.   Z1 .lt. 1.e-6 ) then
-            dothin = iTrack.pos.depth/Zfirst.vec.coszenith
-     *           .gt. 300.
-         else
-            dothin=( iTrack.pos.depth-Zfirst.pos.depth)/
-     *           Zfirst.vec.coszenith .gt. (1000.+ dd)
-         endif
-      else
-         dothin = .true.
-      endif
-      if(dothin) then 
-         iergpn = iTrack.p.fm.p(4)
-         if(iTrack.p.code .eq. kgnuc) then
-            iergpn = iergpn/ iTrack.p.subcode
-         endif
-         aergpn = aTrack.p.fm.p(4)
-         if(aTrack.p.code .eq. kgnuc) then
-            aergpn = aergpn / aTrack.p.subcode
-         endif
-c        We apply thinning if Ethin1>  E
-c        and weight is < Ethin2 
-c           Weight and energy has
-c           E=Ethin(1)/w ; at very low energies
-c          (say < 1GeV ), E<=Ethin(1)/w begins
-c           and brodens  
-         if( aTrack.p.code .le. kelec ) then
-            jj = 1
-         else
-            jj = 3
-         endif
-         if( iTrack.p.code .le. kelec ) then
-            ii = 1
-         else
-            ii = 3
-         endif
-
-         if(iergpn .gt. Ethin(ii) ) then
-            if(aergpn .gt. Ethin(jj)) then
-c                    Both   Ei, Ec> Ethin1; no thinning
-               icon = 0
-               aTrack.wgt = iTrack.wgt
-c         elseif(aergpn .gt. Ethin(2)) then 
-            elseif( aTrack.wgt .lt. Ethin(jj+1)) then
-c              
-               p = aergpn/Ethin(jj)
-               call rndc(u)
-               if(u .lt. p)  then
-                  icon = 0
-                  aTrack.wgt = iTrack.wgt / p
-               else
-                  icon = 1
-               endif
-            else
-               icon = 0
-               aTrack.wgt = iTrack.wgt 
-            endif
-c      elseif(iergpn .gt. Ethin(2) .and.
-c     *       aergpn .gt. Ethin(2) ) then
-         elseif(iTrack.wgt .lt. Ethin(ii+1) .and.
-     *           aTrack.wgt .lt. Ethin(jj+1) ) then
-            p = aergpn/iergpn
-            call rndc(u)
-            if(u .lt. p ) then
-               icon = 0
-               aTrack.wgt = iTrack.wgt / p
-            else
-               icon = 1
-            endif
-         else
-            icon =0
-            aTrack.wgt = iTrack.wgt
-         endif
-      else
-         icon = 0
-         aTrack.wgt = iTrack.wgt
-      endif
-      end
-
 
