@@ -331,7 +331,7 @@ c                  the first collision point. or ptcl goes streight
 c         MovedTrack.t = 0.     ! bug bef uv6.09 ;;; but why commentout ??? 
       endif
       end
-
+C           if you activate the next routine, Epics will have link problem!!!
 #if DEBUG > 0
       subroutine checkstat(str)
       implicit none
@@ -348,27 +348,27 @@ c         MovedTrack.t = 0.     ! bug bef uv6.09 ;;; but why commentout ???
       integer i
       integer nc
       data nc/0/
+
       nc = nc + 1
 c      if(nc .gt. 750) then
-      if( deb) then
+c      if( deb) then
 
-
-            write(*,*) str, ' code=',TrackBefMove.p.code,
+            write(0,*) str, ' code=',TrackBefMove.p.code,
      *         ' cg',TrackBefMove.p.charge,
      *         ' e=', TrackBefMove.p.fm.p(4), ' stat=',MoveStat
-            write(*, *) ' where', TrackBefMove.where
-            write(*,*)  ' h==', TrackBefMove.pos.height,
+            write(0, *) ' where', TrackBefMove.where
+            write(0,*)  ' h==', TrackBefMove.pos.height,
      *       ' d=', TrackBefMove.pos.depth
-            write(*,*) ' cos=',TrackBefMove.vec.coszenith
+            write(0,*) ' cos=',TrackBefMove.vec.coszenith
             do i = 1, NumberOfInte
-               write(*, *) ' process=',IntInfArray(i).process
-               write(*, *) ' thickness=',IntInfArray(i).thickness
-               write(*, *) ' length=',IntInfArray(i).length
+               write(0, *) ' process=',IntInfArray(i).process
+               write(0, *) ' thickness=',IntInfArray(i).thickness
+               write(0, *) ' length=',IntInfArray(i).length
             enddo
-            write(*,*)'  ProcessNo=', ProcessNo
-            write(*,*)  '--------------'
+            write(0,*)'  ProcessNo=', ProcessNo
+            write(0,*)  '--------------'
+c         endif
 c         endif
-         endif
       end
 #endif
 c-------------------------------------------------------
@@ -508,9 +508,9 @@ c///////////////
 c                  some work around for insufficient accuracy 
                if(leng .le. 0.) then
                   leng = 1.d-3
-                  IntInfArray(ProcessNo).thickness = clen2thick(h, 
+                  IntInfArray(i).thickness = clen2thick(h, 
      *              TrackBefMove.vec.coszenith, 
-     *              IntInfArray(ProcessNo).length )
+     *              IntInfArray(i).length )
                endif
             else
                leng = Infty
